/*
* Copyright 2020 NXP
* SPDX-License-Identifier: BSD-3-Clause
*/

#include "peripherals.h"

/******************************************
 * variables
 ******************************************/
uint16_t uw16Cap_Ton;

/*!
 * @brief Initializes the pin functions.
 */
void pin_init(void)
{
	CLOCKGATE_ENABLE(GPIO_A);
	CLOCKGATE_ENABLE(GPIO_B);
	CLOCKGATE_ENABLE(GPIO_C);
	CLOCKGATE_ENABLE(GPIO_E);
	CLOCKGATE_ENABLE(GPIO_F);
	
	// GPIO LEDs off
	GPIOF->PER &= ~(MASK_BIT6|MASK_BIT7|MASK_BIT4|MASK_BIT5); // Set to GPIO mode
	GPIOF->DDR |= (MASK_BIT6|MASK_BIT7|MASK_BIT4|MASK_BIT5); // Set to output mode
	GPIOF->DR |= (MASK_BIT6|MASK_BIT7|MASK_BIT4|MASK_BIT5); // Output 1
	// PWM LEDs off
	GPIOE->PER &= ~(MASK_BIT0|MASK_BIT1|MASK_BIT2|MASK_BIT3|MASK_BIT4|MASK_BIT5); // Set to GPIO mode
	GPIOE->DDR |= (MASK_BIT0|MASK_BIT1|MASK_BIT2|MASK_BIT3|MASK_BIT4|MASK_BIT5); // Set to output mode
	GPIOE->DR &= ~(MASK_BIT0|MASK_BIT1|MASK_BIT2|MASK_BIT3|MASK_BIT4|MASK_BIT5); // Output 0
	
	//SCIO pins
	GPIOC->PER |= GPIO_PER_PE(MASK_BIT11|MASK_BIT12); // Set GPIOC11&12 to peripheral mode
	SIM->GPSCH = (SIM->GPSCH & ~SIM_GPSCH_C11_MASK)|SIM_GPSCH_C11(2); // Set GPIOC11 as TXD1
	SIM->GPSCH = (SIM->GPSCH & ~SIM_GPSCH_C12_MASK)|SIM_GPSCH_C12(2); // Set GPIOC12 as RXD1
}

/*!
 * @brief Initializes internal signal connection through XBAR.
 */
void xbar_init(void)
{
	SIM->IPS0 |= SIM_IPS0_TA0_MASK;
	XBAR_INIT(in_PWMA3_MUX_TRIG0, out_TIMERA_IN0); // select PWMA3A as TA0 secondary input
}

/*!
 * @brief Initializes PWMA3H, PWM3B output generate the sine signal.
 */
void PWMA_3_init(void)
{
	CLOCKGATE_ENABLE(PWMA_SM3);
		
	PWM->SM[3].CTRL2 |= PWM_CTRL2_INDEP_MASK; //independent mode
	PWM->SM[3].INIT = 0;
	PWM->SM[3].VAL0 = 0;
	PWM->SM[3].VAL1 = 4999; // 20Khz @100MHz 2*bus clk at default
	PWM->SM[3].VAL2 = 0;
	PWM->SM[3].VAL3 = 100;
	PWM->SM[3].DISMAP[0] &= ~PWM_DISMAP_DIS0A_MASK; // no fault protection
	PWM->SM[3].DISMAP[1] &= ~PWM_DISMAP_DIS1A_MASK; 
	PWM->FAULT[0].FSTS |= PWM_FSTS_FFLAG_MASK; // Clear fault flags
	PWM->FAULT[1].FSTS |= PWM_FSTS_FFLAG_MASK; // Clear fault flags
	PWM->SM[3].INTEN |= PWM_INTEN_CMPIE(1); // enable PWM3 VAL0 compare interrupt
	PWM->MCTRL |= PWM_MCTRL_LDOK(0x8);
	PWM->OUTEN |= PWM_OUTEN_PWMA_EN(0x8); // Enable output of SM3 PWMB
	
	INTC->IPR9 &= ~INTC_IPR9_PWMA_CMP3_MASK; // enable PWM3 compare interrupt in INTC and configure it's priority
	INTC->IPR9 |= INTC_IPR9_PWMA_CMP3(2);
}

/*!
 * @brief Initializes TA1 to capture PWM ton.
 */
void TA0_init(void)
{
	CLOCKGATE_ENABLE(QTimer_A0); //enable TA1 clock
	
	TMRA->CHANNEL[0].ENBL &= ~TMR_ENBL_ENBL(1); // disable TA0 counter
	TMRA->CHANNEL[0].LOAD = 0;
	TMRA->CHANNEL[0].CNTR = 0;
	//TMRA->CHANNEL[0].COMP1 = 0xFFFF;
	/* CM=6(triggered count mode), PCS=8(bus clock), SCS=0(TMR input0) */
	TMRA->CHANNEL[0].CTRL |= TMR_CTRL_CM(6)|TMR_CTRL_PCS(0x8)|TMR_CTRL_SCS(0x0); 
	/* IPS=0(rising edge triggers count), CAPTURE_MODE=2(load capture on falling edge) */
	TMRA->CHANNEL[0].SCTRL |= TMR_SCTRL_CAPTURE_MODE(2); 
	TMRA->CHANNEL[0].CSCTRL |=  TMR_CSCTRL_TCI_MASK; // reload counter upon receiving a second trigger
	TMRA->CHANNEL[0].DMAr |= TMR_DMA_IEFDE_MASK; // enable DMA request for CAPT when IEF is set
	TMRA->CHANNEL[0].SCTRL &= ~TMR_SCTRL_IEF_MASK; // clear IEF
	TMRA->CHANNEL[0].ENBL |= TMR_ENBL_ENBL(1); // enable TA0 counter 
}

/*!
 * @brief Initializes eDMA for TMR capture register transfer.
 */
void eDMA_init(void)
{
	SIM->CTRL &= ~SIM_CTRL_DMAEBL_MASK;
	SIM->CTRL |= SIM_CTRL_DMAEBL(0x03);//dma module is enabled in all power modes
	
	DMA->CR &= ~DMA_CR_EMLM_MASK; //disable minor loop mapping
	DMA->TCD[0].SADDR = (uint32_t)(&(TMRA->CHANNEL[0].CAPT))<<1;
	DMA->TCD[0].ATTR = 0x0101;// SMOD disable, 16-bit transfer
	DMA->TCD[0].SOFF = 0;
	DMA->TCD[0].NBYTES_MLNO= 0x2;
	DMA->TCD[0].SLAST = 0;
	DMA->TCD[0].DADDR = (uint32_t)(&uw16Cap_Ton)<<1;
	DMA->TCD[0].CITER_ELINKNO = 0x1;
	DMA->TCD[0].DOFF = 0;
	DMA->TCD[0].DLASTSGA = 0;
	DMA->TCD[0].BITER_ELINKNO = 0x1;
	DMA->ERQ |= DMA_ERQ_ERQ0_MASK; //enable DMA request 0
	
}

/*!
 * @brief Configure DMA request source.
 */
void DMAMUX_init(void)
{
	DMAMUX->CHCFG[0] = 0xA0;
}

/*!
 * @brief Initializes SCI0 for FreeMASTER communiction.
 */
void SCI1_init(void)
{
	CLOCKGATE_ENABLE(SCI_1);
	QSCI1->RATE = (((uint16_t)(27)<<3)|1); // 115200
	QSCI1->CTRL3 = 0;
	QSCI1->CTRL1 |= (QSCI_CTRL1_TE_MASK|QSCI_CTRL1_RE_MASK);//enable TX and RX
}

/*!
 * @brief Initializes all used module.
 */
void peripherals_init(void)
{
	pin_init();
	SCI1_init();
	PWMA_3_init();
	TA0_init();
	eDMA_init();
	DMAMUX_init();
	xbar_init();
}
